@@ -1,4 +1,3 @@
-import sys
 import os
 import time
 import subprocess
@@ -7,6 +6,7 @@ import datetime
 import json
 import calendar
 import re
+import pkg_resources
 from viprdata.viprdata import ViprData, FILE_ACCESS_MODE_HEADER, \
   FILE_ACCESS_DURATION_HEADER, FILE_ACCESS_HOST_LIST_HEADER, \
   FILE_ACCESS_USER_HEADER, FILE_ACCESS_START_TOKEN_HEADER, \
@@ -39,8 +39,8 @@ class ViprFileAccess(object):
         response = self.fileaccess_ops["getAccessMode"](namespace, bucket, self.key, self.secret)
         return self._build_bucket_mode_result(response.headers)
 
-    def _set_bucket_mode(self, namespace, bucket, mode, hosts, duration, token, uid):
-        response = self.fileaccess_ops["switchAccessMode"](namespace, bucket, mode, hosts, duration, token, uid, self.key, self.secret)
+    def _set_bucket_mode(self, namespace, bucket, mode, hosts, duration, token, uid, preserve):
+        response = self.fileaccess_ops["switchAccessMode"](namespace, bucket, mode, hosts, duration, token, uid, preserve, self.key, self.secret)
         return self._build_bucket_mode_result(response.headers)
         
     # wait until the mode is pesent
@@ -49,9 +49,12 @@ class ViprFileAccess(object):
         while True:
             response = self.fileaccess_ops["getAccessMode"](namespace, bucket, self.key, self.secret)
             h = response.headers
-            if (h[FILE_ACCESS_MODE_HEADER] and h[FILE_ACCESS_MODE_HEADER].lower() == mode.lower()):
-                # print 'bucket state: ' + mode
-                break
+            if h[FILE_ACCESS_MODE_HEADER]:
+                if (mode and h[FILE_ACCESS_MODE_HEADER].lower() == mode.lower())\
+                        or (mode is None and not h[FILE_ACCESS_MODE_HEADER].startswith('switchingTo')):
+                    # print 'bucket state: ' + mode
+                    break
+
             # print 'sleep 2s'
             time.sleep(2)
         print 'switched to ' + mode
@@ -70,7 +73,7 @@ class ViprFileAccess(object):
         return result
 
 class ViprMount(ViprFileAccess):
-    def __init__(self, api, endpoint, key, secret, namespace, bucket, token, hosts, readonly, uid, duration, parent_dir):
+    def __init__(self, api, endpoint, key, secret, namespace, bucket, token, hosts, readonly, uid, duration, preserve, parent_dir):
         super(ViprMount, self).__init__(api, endpoint, key, secret)
         self.namespace = namespace
         self.bucket = bucket
@@ -79,12 +82,15 @@ class ViprMount(ViprFileAccess):
         self.readonly = readonly
         self.uid = str(uid)
         self.duration = duration
+        self.preserve = preserve
         self.parent_dir = parent_dir
         
     def execute(self):
         print 'exporting %s:%s/%s to %s:%s as %s' % (self.key, self.namespace or '', self.bucket, self.hosts, self.parent_dir, self.uid)
         if (self.token):
             print 'with token ' + self.token
+        if (self.preserve):
+            print 'as originally ingested'
         if (self.readonly):
             print 'read-only'
             mode = 'readOnly'
@@ -175,7 +181,7 @@ class ViprUmount(ViprMount):
                                          mount_info.namespace, mount_info.bucket,
                                          None, mount_info.hosts,
                                          False, mount_info.uid,
-                                         duration, parent_dir)
+                                         duration, False, parent_dir)
         self.end_token = mount_info.token
         self.mount_info = mount_info
 
@@ -202,8 +208,8 @@ class ViprUmount(ViprMount):
         mode = 'disabled'
         self._set_bucket_mode(self.namespace, self.bucket, mode, self.hosts, None,
                               self.end_token, self.uid)
-        # TODO: handle parallel workflows here (might not end up in "disabled")
-        #self._wait_for(self.namespace, self.bucket, mode)
+        # just wait for a non-transitioning state (might not end up in "disabled")
+        self._wait_for(self.namespace, self.bucket)
         
         # delete config file if successful
         self.mount_info.clean()
@@ -321,3 +327,7 @@ class ViprScriptError(Exception):
         
     def __str__(self):
         return repr(self.message)
+
+def cli_version(script):
+    print 'vipr-data v%s' % (pkg_resources.get_distribution('vipr-data').version)
+    print 'vipr-data-cli v%s' % (pkg_resources.get_distribution('vipr-data-cli').version)
